"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateDomainAndModule = exports.getModuleBaseUri = exports.getModuleDomain = exports.moduleToUri = exports.domainToUri = void 0;
const bf_types_1 = require("bf-types");
const utils_1 = require("./utils");
const domainUriMap = new Map();
const moduleUriMap = new Map();
for (const domain of Object.values(bf_types_1.DOMAINS)) {
    domainUriMap.set(domain, utils_1.toLowerCamel(domain));
}
for (const module of Object.values(bf_types_1.ALL_MODULES)) {
    moduleUriMap.set(module, utils_1.toLowerCamel(module));
}
const moduleToDomainMap = new Map();
for (const domainName of Object.keys(bf_types_1.DOMAIN_MODULES)) {
    for (const moduleName of Object.keys(bf_types_1.DOMAIN_MODULES[domainName])) {
        moduleToDomainMap.set(moduleName, domainName);
    }
}
/**
 * Convert a block-5 domain name into a URI formatted string
 *
 * @param domainName The name of a block-5 domain
 */
function domainToUri(domainName) {
    const uri = domainUriMap.get(domainName);
    if (!uri) {
        throw new Error(`there is no URI for the ${domainName} domain`);
    }
    return uri;
}
exports.domainToUri = domainToUri;
/**
 * Convert a block-5 module name into a URI formatted string
 *
 * @param moduleName The name of a block-5 module
 */
function moduleToUri(moduleName) {
    const uri = moduleUriMap.get(moduleName);
    if (!uri) {
        throw new Error(`there is no URI for the ${moduleName} module`);
    }
    return uri;
}
exports.moduleToUri = moduleToUri;
/**
 * Get the domain name associated with a particular module name
 *
 * @param moduleName The name of a block-5 module
 */
function getModuleDomain(moduleName) {
    return moduleToDomainMap.get(moduleName) || null;
}
exports.getModuleDomain = getModuleDomain;
/**
 * Get the base URI associated with a particular block-5 module name
 *
 * @param moduleName The name of a block-5 module
 */
function getModuleBaseUri(moduleName, uri) {
    const domainName = getModuleDomain(moduleName);
    return `${domainToUri(domainName)}/${moduleToUri(moduleName)}${uri ? `/${uri}` : ''}`;
}
exports.getModuleBaseUri = getModuleBaseUri;
/**
 * Ensure that both a domain name and module name are valid
 *
 * @param domainName The name of a block-5 domain
 * @param moduleName The name of a block-5 module
 */
function validateDomainAndModule(domainName, moduleName) {
    if (!bf_types_1.DOMAINS[domainName]) {
        throw new Error(`${domainName} is not a valid entity domain`);
    }
    if (!bf_types_1.ALL_MODULES[moduleName]) {
        throw new Error(`${moduleName} is not a valid entity module`);
    }
    const domainModules = bf_types_1.DOMAIN_MODULES[domainName];
    if (typeof domainModules[moduleName] !== 'string') {
        throw new Error(`${module} is not a module of the ${utils_1.toDisplay(domainName)}`);
    }
}
exports.validateDomainAndModule = validateDomainAndModule;
